import mock
from torch_geometric.nn import MetaLayer


def test_meta_layer():
    assert MetaLayer().__repr__() == ('MetaLayer(\n'
                                      '    edge_model=None,\n'
                                      '    node_model=None,\n'
                                      '    global_model=None\n'
                                      ')')

    edge_model = mock.MagicMock()
    node_model = mock.MagicMock()
    global_model = mock.MagicMock()

    for em in (edge_model, None):
        for nm in (node_model, None):
            for gm in (global_model, None):
                model = MetaLayer(em, nm, gm)
                out = model(
                    mock.MagicMock(),
                    edge_index=("row", "col"),
                    edge_attr="edge_attr",
                    u="u",
                    batch="batch")

                assert isinstance(out, tuple) and len(out) == 3

                if em is not None:
                    em.assert_called_once()
                if nm is not None:
                    nm.assert_called_once()
                if gm is not None:
                    gm.assert_called_once()

                edge_model.reset_mock()
                node_model.reset_mock()
                global_model.reset_mock()
